
----------------------------------------------
-- Variables
----------------------------------------------

g_BBar = {};					-- Main storage object

-- Static Veriables
g_BBar.BASEBARSZ = 78;
g_BBar.BASEBARMINSZ = 58;
g_BBar.TOTALITEMS = 33;			-- Total number of items including titles (size of BBarObjects_Shown array)
g_BBar.MINCURRENCYID = 10;		-- Smallest possible currency ID (below this are title IDs)
g_BBar.MONEYID = 11;			-- ID of the money object
g_BBar.MONEYPOS = 2;			-- Position in BBarObjects of the money object
g_BBar.BARRESIZESTEP = 20;		-- Higher number = faster movement
g_BBar.BARREPSHIFT = 28;		-- Distance the rep bars are shifted to the left

-- Currency and custom items
g_BBar.BBarObjects = {1, 11, 2, 395, 396, 392, 390, 3, 61, 361, 698, 402, 81, 4, 515, 241, 416, 391, 614, 615, 697, 5, 384, 398, 394, 385, 397, 393, 400, 401, 676, 677, 6};
g_BBar.BBarObjects_Shown = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
g_BBar.BBarObjectsItems = {};
g_BBar.BBarObjectsItems_Shown = {};
g_BBar.BBarObjectsReps_Faction = {};	-- List of displayed faction IDs

-- Other
g_BBar.Setup = 0;
g_BBar.OnetimeEventUpdate = 0;
g_BBar.MoneyFrameObject = nil;
g_BBar.BBarTitleRegion = nil;
g_BBar.CurrencyScrollFrame = nil;
g_BBar.RepScrollFrame = nil;

g_BBar.CurrentSettingsTab = 0;
g_BBar.LastSettingsItem = nil;
g_BBar.DefaultSettingsheight = 0;
g_BBar.UpdateBarPosition = 0;		-- Skip updating it the first time or it'll be positioned wrong
g_BBar.PosStorage = {};
g_BBar.TooltipStorage = {};

-- Resize animation
g_BBar.rsCurrentSize = 1;		-- 0 = Small, 1 = Large
g_BBar.rsAnimating = 0;			-- 1 = Shrinking or Growing
g_BBar.rsAnimatingSize = 0;		-- Current width
g_BBar.rsStoredWidth = 0;


----------------------------------------------
-- Functions
----------------------------------------------


SLASH_BBAR_RESET1 = '/bbar';
SlashCmdList["BBAR_RESET"] = function()
	_G["BBarFrame"]:ClearAllPoints();
	_G["BBarFrame"]:SetPoint("CENTER", 0, 0);
	_G["BBarSettingsFrame"]:ClearAllPoints();
	_G["BBarSettingsFrame"]:SetPoint("CENTER", 0, 0);
end


function BBar_OnLoad(self)

	g_BBar.BBarTitleRegion = self:CreateTitleRegion();

	-- Store the money frame object and remove the scripts
	g_BBar.MoneyFrameObject = _G[self:GetName().."MoneyFrame"];
	_G[g_BBar.MoneyFrameObject:GetName().."CopperButton"]:SetScript("OnClick", nil);
	_G[g_BBar.MoneyFrameObject:GetName().."SilverButton"]:SetScript("OnClick", nil);
	_G[g_BBar.MoneyFrameObject:GetName().."GoldButton"]:SetScript("OnClick", nil);

	-- Hook the container item clicking for right click adding.
	hooksecurefunc("HandleModifiedItemClick", BBar_HandleModifiedItemClick);

	ValuesToString();	-- Setup defaults

	_G["BBarSettingsFrameTab1"]:SetChecked(1);
	_G["BBarSettingsFrameTab2"]:SetChecked(nil);

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_LOGOUT");
	self:RegisterEvent("PLAYER_MONEY");
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
	self:RegisterEvent("UPDATE_FACTION");

end


function BBar_OnEvent(self, event, ...)

	if( event == "PLAYER_ENTERING_WORLD" ) then

		if(g_BBar.Setup == 1)	then	return	end;
		MoneyFrame_UpdateMoney(g_BBar.MoneyFrameObject);
		BBar_UpdateBar();
		BBar_UpdateReputations();

		BBar_BuildSettingsFrame();
		g_BBar.CurrencyScrollFrame:Show();
		g_BBar.RepScrollFrame:Hide();
		g_BBar.Setup = 1;
	end
	if( event == "PLAYER_LOGOUT" ) then
		local BPs = {};
		BPs[1], BPs[2], BPs[3], BPs[4], BPs[5] = _G["BBarFrame"]:GetPoint();
		BBarPos = BPs;
	end
	if( event == "PLAYER_MONEY" ) then
		MoneyFrame_UpdateMoney(g_BBar.MoneyFrameObject);
		BBar_UpdateBar();
	end
	if( event == "BAG_UPDATE" ) or ( event == "CURRENCY_DISPLAY_UPDATE" ) then
		BBar_UpdateBar();		-- Would be nice to know what changed on bag update so it only updates if needed
	end
	if( event == "UPDATE_FACTION" ) then
		BBar_UpdateReputations();		-- Would be nice to know what changed so it only updates if needed
	end
	if( event == "VARIABLES_LOADED" ) then
	--	ValuesToString();	-- Reset saved string
		StringToValues();	-- parse variable string from storage

		-- Position the window
		g_BBar.OnetimeEventUpdate = 1;

		-- Set up the window lock
		if(BBarWindowLock == nil)	then   BBarWindowLock = 0;	end
		if(BBarWindowLock > 0) then		_G["BBarSettingsFrameLockCheck"]:SetChecked(true);
		else							_G["BBarSettingsFrameLockCheck"]:SetChecked(false);
		end
		BBar_ToggleMoving(_G["BBarSettingsFrameLockCheck"]);
	end
end


function BBar_ToggleSettings()

	if(_G["BBarSettingsFrame"]:IsShown()) then
			HideUIPanel(_G["BBarSettingsFrame"]);
	else	ShowUIPanel(_G["BBarSettingsFrame"]);
	end
end


function BBar_ToggleSize()

	-- Large -> Small
	if(g_BBar.rsCurrentSize == 1) then
		_G["BBarFrameResizing"]:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
		_G["BBarFrameResizing"]:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");

		g_BBar.rsStoredWidth = _G["BBarFrame"]:GetWidth();	-- Store the current width
		g_BBar.rsAnimatingSize = g_BBar.rsStoredWidth;

		-- Hide everything
		_G["BBarFrameMoneyFrame"]:Hide();
		_G["BBarFrameSettings"]:Hide();
		
		-- Currencies
		local i = 3;	-- Start on 3 to skip the money frame
		while (g_BBar.BBarObjects[i] ~= nil) do
			if(g_BBar.BBarObjects[i] > g_BBar.MINCURRENCYID) then
				BBar_AddNewCurrencyToken(0, 0, g_BBar.BBarObjects[i], 0);
			end
			i = i + 1;
		end

		-- Custom items
		for i = 1, getn(g_BBar.BBarObjectsItems), 1 do
			if(g_BBar.BBarObjectsItems[i] > 0) then
				BBar_AddNewCurrencyToken(0, 0, -g_BBar.BBarObjectsItems[i], 0);
			end
		end

		-- Reputations
		for pos, values in pairs(g_BBar.BBarObjectsReps_Faction) do
			local outWidth, outObj = BBar_AddNewReputationBar(0, 0, values[1], 0, 0);
		end

	-- Small -> Large
	else
		_G["BBarFrameResizing"]:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");
		_G["BBarFrameResizing"]:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");
		g_BBar.rsAnimatingSize = g_BBar.BASEBARMINSZ;
		-- When animation is finished, it'll update the bar to draw all currencies
	end

	-- Start the resize
	g_BBar.rsCurrentSize = (1 - g_BBar.rsCurrentSize);
	g_BBar.rsAnimating = 1;	-- Start animating
end

function BBar_UpdateEvent(self)

	-- Update the window
	if(g_BBar.OnetimeEventUpdate == 1) then
		if(BBarPos ~= nil)	then
			_G["BBarFrame"]:ClearAllPoints();
			_G["BBarFrame"]:SetPoint(BBarPos[1], BBarPos[2], BBarPos[3], BBarPos[4], BBarPos[5]);
		end
		g_BBar.OnetimeEventUpdate = 0;
	end

	-- Update the animation
	if(g_BBar.rsAnimating ~= 1)	then	return	end;

	StoreBBPos();
	-- Growing
	if(g_BBar.rsCurrentSize == 1) then
		if(g_BBar.rsAnimatingSize >= g_BBar.rsStoredWidth) then
			self:SetWidth(g_BBar.rsStoredWidth);
			g_BBar.rsAnimating = 0;	-- Stop animation
			_G["BBarFrameSettings"]:Show();
			BBar_UpdateBar();	-- Make sure it's all as it should be
		else
			g_BBar.rsAnimatingSize = g_BBar.rsAnimatingSize + g_BBar.BARRESIZESTEP;
			if(g_BBar.rsAnimatingSize > g_BBar.rsStoredWidth) then	g_BBar.rsAnimatingSize = g_BBar.rsStoredWidth;	end
			self:SetWidth(g_BBar.rsAnimatingSize);
		end
	else	-- Shrinking
		if(g_BBar.rsAnimatingSize <= g_BBar.BASEBARMINSZ) then
			self:SetWidth(g_BBar.BASEBARMINSZ);
			g_BBar.rsAnimating = 0;	-- Stop animation
		else
			g_BBar.rsAnimatingSize = g_BBar.rsAnimatingSize - g_BBar.BARRESIZESTEP;
			if(g_BBar.rsAnimatingSize < g_BBar.BARRESIZESTEP) then	g_BBar.rsAnimatingSize = g_BBar.BARRESIZESTEP;	end
			self:SetWidth(g_BBar.rsAnimatingSize);
		end
	end
	UpdateBBPos();
end


function BBar_ShowCurrencyTooltip(self)
	if((self == nil) or (self.currencyID == nil)) then	return	end
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT");
	if(self.currencyID > g_BBar.MINCURRENCYID)	then	GameTooltip:SetCurrencyByID(self.currencyID);	end
	if(self.currencyID < 0)						then	GameTooltip:SetItemByID(abs(self.currencyID));	end
end


function BBar_ShowResizeTooltip(self)

	GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT");
	if(g_BBar.rsCurrentSize == 1) then	GameTooltip:SetText("Shrink Bar", 1, 1, 1);
	else	GameTooltip:SetText("Enlarge Bar", 1, 1, 1);
	end
	GameTooltip:Show();
end


function BBar_ShowReputationTooltip(self)

	local name =		_G[self:GetName().."FactionName"]:GetText();
	local value =		_G[self:GetName().."Value"]:GetText();
	local standing =	_G[self:GetName().."Standing"]:GetText();

	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetText(name, 1, 1, 1);
	GameTooltip:AddLine(value, 1, 1, 1);
	GameTooltip:AddLine(standing, 1, 1, 1);
	GameTooltip:Show();
end


---------------------------


function BBar_UpdateBar()

	if(g_BBar.UpdateBarPosition == 1)	then	StoreBBPos();	end

	-- Add new tokens
	local priorObj = nil;
	local extraWidth = 0;
	local self = _G["BBarFrame"];

	if(g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] > 0) then
			_G["BBarFrameMoneyFrame"]:Show();
	else	_G["BBarFrameMoneyFrame"]:Hide();
	end

	-- Currencies
	local i = 3;	-- Start on 3 to skip the money frame
	while (g_BBar.BBarObjects[i] ~= nil) do
		if(g_BBar.BBarObjects[i] > g_BBar.MINCURRENCYID) then
			local outWidth, outObj = BBar_AddNewCurrencyToken(self, priorObj, g_BBar.BBarObjects[i], g_BBar.BBarObjects_Shown[i]);
			if(outWidth > 0)	then	extraWidth = extraWidth + outWidth;		end
			if(outObj ~= 0)		then	priorObj = outObj;						end
		end
		i = i + 1;
	end

	-- Custom items
	for i = 1, getn(g_BBar.BBarObjectsItems), 1 do
		if(g_BBar.BBarObjectsItems[i] > 0) then
			local outWidth, outObj = BBar_AddNewCurrencyToken(self, priorObj, -g_BBar.BBarObjectsItems[i], g_BBar.BBarObjectsItems_Shown[i]);
			if(outWidth > 0)	then	extraWidth = extraWidth + outWidth;		end
			if(outObj ~= 0)		then	priorObj = outObj;						end
		end
	end
	
	local moneyWidth = 0;
	if(g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] > 0)	then	moneyWidth = _G["BBarFrameMoneyFrame"]:GetWidth();	end
	local TotalWidth = g_BBar.BASEBARSZ + moneyWidth + extraWidth;
	self:SetWidth(TotalWidth);

	BBar_UpdateReputations();

	if(g_BBar.UpdateBarPosition == 1)	then	UpdateBBPos();	end
	g_BBar.UpdateBarPosition = 1;	-- Make it update on all subsequent passes
end


function BBar_UpdateReputations()

	-- Update the reputation bars
	local bar = _G["BBarFrame"];
	local TotalWidth = (bar:GetWidth() - g_BBar.BARREPSHIFT);		-- Remove the spacing on the left 
	local priorObj = nil;
	
	-- Because of dodgy lua i need to extract the data and sort the list manually
	local count = 0;
	local tempStore = {};
	for pos, values in pairs(g_BBar.BBarObjectsReps_Faction) do
		tinsert(tempStore, {pos, values[1], values[2]});
		count = count + 1;
	end
	if(#tempStore > 1)	then	sort(tempStore, function(a, b) return a[1] < b[1] end);	end;

	for i = 1, count do
		local outObj = BBar_AddNewReputationBar(bar, priorObj, tempStore[i][2], tempStore[i][3], TotalWidth);
		if(outObj ~= 0)	then	priorObj = outObj;	end
	end
end


function BBar_AddNewCurrencyToken(self, priorObj, currencyID, shown)

	local thisObj = _G["BBarCurrencyToken"..currencyID];

	-- Check if this button should be made or shown
	if(shown == 0) then
		if(thisObj) then thisObj:Hide(); end
		return 0, 0;
	end

	-- If this button doesn't exist yet, create it
	if( not thisObj ) then
		thisObj = CreateFrame("BUTTON", "BBarCurrencyToken"..currencyID, self, "BBarCurrencyFrameTemplate");
	else	thisObj:ClearAllPoints();
	end
	_G[thisObj:GetName().."Spacer"]:Show();		-- Incase it was hidden by being the first item earlier

	-- Get this currency info and update the button
	local objWidth = 0;
	local name, count, icon;
	if(currencyID > g_BBar.MINCURRENCYID) then
		name, count, icon = GetCurrencyInfo(currencyID);
		icon = "Interface\\Icons\\"..icon;
	elseif(currencyID < 0) then		-- Item
		name = GetItemInfo(abs(currencyID));
		count = GetItemCount(abs(currencyID), true);
		icon = GetItemIcon(abs(currencyID));
	end

	if( name and name ~= "" ) then
		thisObj:SetText(count);
		thisObj:SetNormalTexture(icon);
		thisObj.currencyID = currencyID;
		thisObj:Show();
		objWidth = thisObj:GetTextWidth() + thisObj:GetWidth() + 28;
		if(priorObj ~= nil)	then	thisObj:SetPoint("LEFT", priorObj, "RIGHT", priorObj:GetTextWidth() + 28, 0);
		else
			if(g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] > 0) then
				thisObj:SetPoint("LEFT", _G["BBarFrameMoneyFrame"], "RIGHT", 13, 0);
			else
				_G[thisObj:GetName().."Spacer"]:Hide();
				thisObj:SetPoint("LEFT", _G["BBarFrame"], "LEFT", 48, -1);
			end
		end
	else	thisObj:Hide();	end
	return objWidth, thisObj;
end


function BBar_AddNewReputationBar(self, priorObj, factionID, shown, width)

	local thisObj = _G["BBarReputationFrame"..factionID];

	-- Check if this frame should be made or shown
	if(shown == 0) then
		if(thisObj) then thisObj:Hide(); end
		return 0;
	end

	-- If this frame doesn't exist yet, create it
	if( not thisObj ) then
		thisObj = CreateFrame("FRAME", "BBarReputationFrame"..factionID, self, "BBarReputationFrameTemplate");
	else	thisObj:ClearAllPoints();
	end
	thisObj:SetWidth(width);
--	_T(tostring(width));
	local factionTitle =		_G["BBarReputationFrame"..factionID.."FactionName"];
	local factionValue =		_G["BBarReputationFrame"..factionID.."Value"];
	local factionStanding =		_G["BBarReputationFrame"..factionID.."Standing"];
	local factionBarFill =		_G["BBarReputationFrame"..factionID.."PercentageBar"];

	-- Hide things if not enough space
	if(width < 370) then	factionValue:Hide();
	else					factionValue:Show();
	end
	if(width < 200) then	factionStanding:Hide();
	else					factionStanding:Show();
	end

	-- Get this faction info and update the frame
	local name, _, standingID, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(factionID);
	factionTitle:SetText(name);
	local factionStandingtext = GetText("FACTION_STANDING_LABEL"..standingID, UnitSex("player"));
	
	-- Check if this is a friendship faction (members of the Tillers)
	local friendID, _, friendMaxRep, _, _, friendTextLevel, friendThresh = GetFriendshipReputationByID(factionID);
	if(friendID ~= nil) then
		factionStandingtext = friendTextLevel;
		barMax = friendMaxRep - friendThresh;
		barValue = barValue - friendThresh;
	else
		barMax = barMax - barMin;
		barValue = barValue - barMin;
		if(barValue < 0) then	barValue = (barMax - barValue);	end
	end
	if(barValue == 0) then
			factionBarFill:Hide();	-- Supplying 0 causes max size
	else
		factionBarFill:Show();
		factionBarFill:SetWidth((((width - 2) / abs(barMax)) * abs(barValue)) - 1);
	end

	factionValue:SetText(tostring(barValue).."/"..tostring(barMax));
	factionStanding:SetText(factionStandingtext);
	local color = FACTION_BAR_COLORS[standingID];
	factionBarFill:SetVertexColor(color.r, color.g, color.b, 1);
	thisObj:Show();

	-- Position
	if(priorObj ~= nil)	then	thisObj:SetPoint("TOPLEFT", priorObj, "BOTTOMLEFT", 0, 1);
	else						thisObj:SetPoint("TOPLEFT", _G["BBarFrame"], "BOTTOMLEFT", g_BBar.BARREPSHIFT, 1);
	end
	return thisObj;
end


---------------------------


function StoreBBPos()

	-- Get the location so we can keep it left aligned when resized.
	g_BBar.PosStorage[1], g_BBar.PosStorage[2], g_BBar.PosStorage[3], g_BBar.PosStorage[4], g_BBar.PosStorage[5] = _G["BBarFrame"]:GetPoint();
	g_BBar.PosStorage[6] = _G["BBarFrame"]:GetWidth();
end

function UpdateBBPos()

	-- Only update if it was stored correctly
	if(g_BBar.PosStorage[1] == nil) then	return	end;

	local Width = _G["BBarFrame"]:GetWidth();
	
	if(strfind(g_BBar.PosStorage[1], "LEFT") ~= nil) then
		g_BBar.PosStorage[4] = g_BBar.PosStorage[4];		-- Do nothing
	elseif(strfind(g_BBar.PosStorage[1], "RIGHT") ~= nil) then
		g_BBar.PosStorage[4] = (g_BBar.PosStorage[4] - g_BBar.PosStorage[6]) + Width;
	else	-- TOP CENTER BOTTOM
		g_BBar.PosStorage[4] = (g_BBar.PosStorage[4] - (g_BBar.PosStorage[6] / 2)) + (Width / 2);
	end
	_G["BBarFrame"]:SetPoint(g_BBar.PosStorage[1], g_BBar.PosStorage[2], g_BBar.PosStorage[3], g_BBar.PosStorage[4], g_BBar.PosStorage[5]);
	g_BBar.PosStorage[1] = nil;
end


-- Convert between arrays and strings for saving
function ValuesToString()	-- Saving

	BBarShownCurrency = "";
	for i = 1, g_BBar.TOTALITEMS, 1 do
		BBarShownCurrency = BBarShownCurrency..tostring(g_BBar.BBarObjects_Shown[i]);
	end

	BBarCustomItems = "";
	BBarShownItems = "";
	for i = 1, getn(g_BBar.BBarObjectsItems), 1 do
		if(g_BBar.BBarObjectsItems[i] > 0) then
			BBarCustomItems = BBarCustomItems..tostring(g_BBar.BBarObjectsItems[i]).." ";
			BBarShownItems = BBarShownItems..tostring(g_BBar.BBarObjectsItems_Shown[i]);
		end
	end

	-- Store the reputations as <position> <faction>
	BBarReputations = "";
	for pos, values in pairs(g_BBar.BBarObjectsReps_Faction) do
		if(values[2] > 0) then		-- Only save the ones shown
			BBarReputations = BBarReputations..tostring(pos).." ";
			BBarReputations = BBarReputations..tostring(values[1]).." ";
		end
	end
end

function StringToValues()	-- Loading

	for i = 1, g_BBar.TOTALITEMS, 1 do
		g_BBar.BBarObjects_Shown[i] = tonumber(strbyte(BBarShownCurrency, i)) - 48;	-- Convert ascii code to number
	end

	local items = { strsplit(" ", BBarCustomItems) };
	for i = 1, getn(items) - 1, 1 do	-- -1 as last item is always invalid
		if(tonumber(items[i]) > 0) then
			g_BBar.BBarObjectsItems[i] = tonumber(items[i]);
			g_BBar.BBarObjectsItems_Shown[i] = tonumber(strbyte(BBarShownItems, i)) - 48;	-- Convert ascii code to number
		end
	end
	
	local count = 0;
	local pos = 0;
	items = { strsplit(" ", BBarReputations) };
	for i = 1, getn(items) - 1, 1 do	-- -1 as last item is always invalid
		if(tonumber(items[i]) > 0) then
			if(count == 0) then	pos = tonumber(items[i]);
			else	g_BBar.BBarObjectsReps_Faction[pos] = {tonumber(items[i]), 1};
			end
			count = (1 - count);
		end
	end
end

function _T(text)
	DEFAULT_CHAT_FRAME:AddMessage(text, 1, 1, 1);
end
